@@ -786,3 +786,10 @@ NULL
   }
   res
 }
+
+
+# Principal component methods with Fcatominer
+f_pca <- function(X, graph = FALSE){
+  FactoMineR::PCA(X, graph = FALSE)
+}
+
